<?php

namespace Drupal\ses_email_bounce\Controller;

/**
 * @file
 * Contains \Drupal\ses_email_bounce\Controller\SesEmailController.
 */

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Database\Connection;
use Drupal\webform\Plugin\WebformHandler\EmailWebformHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Config\ConfigFactory;

/**
 * Controller ses email bounce class.
 */
class SesEmailController extends ControllerBase {

  /**
   * Configuration Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;
  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type manager node storage object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $nodeStorage;

  /**
   * The entity type manager webform storage object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $webformStorage;

  /**
   * The database connection service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs a new SettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The factory for configuration objects.
   *
   *   Constructs a Drupal\rest\Plugin\ResourceBase object.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   A entity type manager.
   * @param \Drupal\Core\Database\Connection $connection
   *   A connection .
   */
  public function __construct(
    ConfigFactory $configFactory,
    array $serializer_formats,
    EntityTypeManagerInterface $entity_type_manager,
    Connection $connection
  ) {
    $this->configFactory = $configFactory;
    $this->entityTypeManager = $entity_type_manager;
    $this->nodeStorage = $this->entityTypeManager->getStorage('node');
    $this->webformStorage = $this->entityTypeManager->getStorage('webform');
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->getParameter('serializer.formats'),
      $container->get('entity_type.manager'),
      $container->get('database'),
    );
  }

  /**
   * Callback for `ses-api/get` API method.
   */
  public function sesSubscribtion(Request $request) {

    $config = $this->configFactory->get('ses_email_bounce.settings');
    $ses_api_status = $config->get('ses_api_status');
    if ($ses_api_status == 0) {
      $response = new JsonResponse(
        [
          'status' => 'ses module not enabled',
        ],
        200
      );
      return $response;
    }

    $header_content_type = $config->get('header_content_type');
    $header_x_amz_sns_topic_arn = $config->get('header_x_amz_sns_topic_arn');
    $ses_subscribtion_log_relative_path = $config->get('ses_subscribtion_log_relative_path');
    $ses_notification_log_relative_path = $config->get('ses_notification_log_relative_path');
    $base_domain_conf = $config->get('base_domain');
    $ses_node_map = $config->get('ses_node_map');
    $email_field_list = [];
    if ($ses_node_map != '') {
      $email_field_list = explode(",", $ses_node_map);
    }
    $exclude_webform_handlers = [];

    $ses_webform_handler_exlude = $config->get('ses_webform_handler_exlude');

    if ($ses_webform_handler_exlude != '') {
      $exclude_webform_handlers = explode(",", $ses_webform_handler_exlude);
    }
    $base_domain = $request->server->get('HTTP_HOST', NULL);
    $datalog = '';
    $data_subscribtion_log = '';
    if ($base_domain == $base_domain_conf) {
      $datalog = $ses_notification_log_relative_path;
      $data_subscribtion_log = $ses_subscribtion_log_relative_path;
    }
    else {
      $domain_arr = [
        'status' => 'mismatch',
        'confic_base_domain' => $base_domain_conf,
        'request_base_domain' => $base_domain,
      ];
      $response = new JsonResponse(
        $domain_arr,
        200
          );
      $datajson = json_encode($domain_arr);
      $datajson = date('d-m-Y, h i') . "::" . $datajson . PHP_EOL;
      file_put_contents("ses-logs/" . $base_domain_conf, $datajson, FILE_APPEND);
      return $response;
    }

    $data = [];
    if (0 === strpos($request->headers->get('Content-Type'), $header_content_type)) {
      if ($request->headers->get('x-amz-sns-topic-arn') == $header_x_amz_sns_topic_arn) {
        if ($request->headers->get('x-amz-sns-message-type') == 'SubscriptionConfirmation') {
          $flag = TRUE;
          $data = json_decode($request->getContent(), TRUE);
          // SubscribeURL.
          $subscribeURL = $data['SubscribeURL'];
          file_put_contents($data_subscribtion_log, $subscribeURL, FILE_APPEND);
          $response = new JsonResponse(
            [
              'success' => $flag,
            ],
            200
          );
          return $response;
        }
        elseif ($request->headers->get('x-amz-sns-message-type') == 'Notification') {
          $reqbody = preg_replace('/\s\s+/', '', $request->getContent());
          $reqbody = json_decode($reqbody, TRUE, JSON_UNESCAPED_SLASHES);
          $reqbody = json_decode($reqbody['Message'], TRUE, JSON_UNESCAPED_SLASHES);
          $data['Message'] = $reqbody;
          if (isset($data['Message']['notificationType']) && $data['Message']['notificationType'] == 'Bounce') {

            if (isset($data['Message']['bounce'])) {

              if ($data['Message']['bounce']['bounceType'] == 'Permanent' && $data['Message']['bounce']['bounceSubType'] == 'Suppressed') {

                if (isset($data['Message']['bounce']['bouncedRecipients'])) {
                  if ($data['Message']['bounce']['bouncedRecipients'][0]['action'] == 'failed') {

                    $ses_bounce_email = $data['Message']['bounce']['bouncedRecipients'][0]['emailAddress'];
                    $node_list = $this->prepareNodeList($ses_bounce_email, $email_field_list);

                    $node_chunk = array_chunk($node_list, 10);
                    $nodesData = [];
                    foreach ($node_chunk as $node_chunk_value) {
                      $nodesResponse = $this->processNodes($node_chunk_value, $ses_bounce_email, $email_field_list);
                      if ($nodesResponse) {
                        $nodesData[] = $nodesResponse;
                      }
                    }

                    // Webform list and process.
                    $form_list = $this->webformStorage->loadMultiple();

                    $webform_chunk = array_chunk($form_list, 1);
                    $webForm = [];
                    foreach ($webform_chunk as $webform_chunk_value) {
                      $webresponse = $this->processWebforms($webform_chunk_value, $ses_bounce_email, $exclude_webform_handlers);
                      if (count($webresponse) > 0) {
                        $webForm[] = $webresponse;
                      }
                    }
                    $data = [
                      'email' => $ses_bounce_email,
                      'nodesData' => $nodesData,
                      'webForm' => $webForm,
                      'SigningCertURL' => $data['SigningCertURL'],
                      'UnsubscribeURL' => $data['UnsubscribeURL'],
                      'Timestamp' => $data['Timestamp'],
                    ];
                    $datajson = json_encode($data);
                    $datajson = date('d-m-Y, h i') . "::" . $datajson . PHP_EOL;
                    file_put_contents($datalog, $datajson, FILE_APPEND);
                    $response = new JsonResponse(
                      [
                        'success' => TRUE,
                        'email' => $ses_bounce_email,
                        'nodesData' => $nodesData,
                        'webForm' => $webForm,
                      ],
                      200
                    );
                    return $response;
                  }
                }
              }
            }
          }
        }
      }
    }
    return new JsonResponse($data);
  }

  /**
   * Prepare Node List.
   */
  public function prepareNodeList($email, $email_field_list) {
    // $email_field_list = ['field_associated_email', 'field_map_to'];
    $return_node_list = [];
    // Search for email in each field database table.
    foreach ($email_field_list as $field_name) {
      $query_result = $this->connection->select('node__' . $field_name, 'data')
        ->fields('data', [$field_name . '_value', 'entity_id'])
        ->condition('data.' . $field_name . '_value', '%' . $email . '%', 'LIKE')
        ->execute()->fetchAll();
      if (!empty($query_result)) {
        foreach ($query_result as $result) {
          $return_node_list[] = $result->entity_id;
        }
      }
    }
    return $return_node_list;
  }

  /**
   * Helper function to process and resave node list.
   */
  public function processNodes($options, $email, $email_field_list) {
    $node_storage = $this->nodeStorage;
    $tempNodes = [];
    foreach ($options as $nid) {
      $node_object = $node_storage->load($nid);
      foreach ($email_field_list as $email_field) {
        if ($node_object->hasField($email_field)) {
          $field_name = $email_field;
        }
      }
      if (!empty($field_name)) {
        $mails = $node_object->get($field_name)->getValue();
        // Hold emails before remove.
        $tempmails = $mails;
        foreach ($mails as $key => $mail) {
          // Check for case insensitive mail ID and unset from field value.
          if (stripos($mail['value'], $email) !== FALSE) {
            unset($mails[$key]);
          }
        }
        $node_object->set($field_name, $mails);
        $tempNodes['nid'] = $nid;
        $tempNodes['mails'] = $tempmails;
        $node_object->save();
      }
    }
    return $tempNodes;
  }

  /**
   * Helper function to process and resave webform handler configurations.
   */
  public function processWebforms($options, $email, $exclude_webform_handlers) {
    $entity_list_string = [];
    $form_object = $options[0];
    $webform_plugins = $form_object->getPluginCollections();
    $instances = $webform_plugins['handlers']->getInstanceIds();
    foreach ($instances as $handler_id) {
      // Filter handlers and remove handlers of custom implementations.
      if (!in_array($handler_id, $exclude_webform_handlers)) {
        $handler = $form_object->getHandler($handler_id);
        if ($handler instanceof EmailWebformHandler) {
          $handler_config = $handler->getConfiguration();
          // Update form only if the selected email id exist case insensitive.
          if (stripos($handler_config['settings']['to_mail'], $email) !== FALSE) {
            $handler_mail_list = explode(',', $handler_config['settings']['to_mail']);
            $temp_handler_mail_list = $handler_mail_list;
            foreach ($handler_mail_list as $key => $mail) {
              if (stripos($mail, $email) !== FALSE) {
                unset($handler_mail_list[$key]);
                $handler_mail_list_new = implode(',', $handler_mail_list);
                $handler_config['settings']['to_mail'] = $handler_mail_list_new;
              }
            }
            $handler->setConfiguration($handler_config);
            $form_object->updateWebformHandler($handler);
            if (!empty($form_object) && isset($form_object)) {
              if ($form_object->id() != '') {
                $entity_list_string['webform_id'] = $form_object->id();
                $entity_list_string['Handler_id'] = $handler_id;
                $entity_list_string['to_email'] = $handler_config['settings']['to_mail'];
                $entity_list_string['emails'] = $temp_handler_mail_list;
                $entity_list_string['email_count'] = count($temp_handler_mail_list);
              }
            }
          }
        }
      }
    }
    return $entity_list_string;
  }

}
