<?php

namespace Drupal\ses_email_bounce\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Define a form that configures forms module settings.
 */
class AwsSesConfigurationForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ses_email_bounce_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'ses_email_bounce.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ses_email_bounce.settings');
    $form['ses_api_status'] = [
      '#type' => 'checkbox',
      '#title' => 'Enable',
      '#default_value' => $config->get('ses_api_status'),
      '#description' => $this->t('Enable/Deabled log and email removal from noded and webform handler'),
    ];
    $form['header_content_type'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Ses Header Content type'),
      '#default_value' => $config->get('header_content_type'),
      '#disabled' => TRUE,
      '#description' => $this->t('AWS SES request header content type.'),
    ];
    $form['header_x_amz_sns_topic_arn'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Ses X amz sns topic arn'),
      '#default_value' => $config->get('header_x_amz_sns_topic_arn'),
      '#description' => $this->t('Ses X amz sns topic arn will be provided by AWS team'),
    ];
    $form['base_domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base Domain'),
      '#default_value' => $config->get('base_domain'),
      '#description' => $this->t('base domain without https exp: www.example.com,hcltech.com'),
    ];
    $form['ses_subscribtion_log_relative_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subscribtion log file path'),
      '#default_value' => $config->get('ses_subscribtion_log_relative_path'),
      '#description' => $this->t('Subscription log file contain subscription url to verify ses subscription.make sure file is exist in folder.'),
    ];
    $form['ses_notification_log_relative_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Notification log file path'),
      '#default_value' => $config->get('ses_notification_log_relative_path'),
      '#description' => $this->t('Will contain bounce log file if bounce type is  Permanent and bounce sub type is Suppressed. make sure file is exist in folder..'),
    ];
    $form['ses_node_map'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Email in Nodes'),
      '#default_value' => $config->get('ses_node_map'),
      '#description' => $this->t('add field machine name those field contain emails, exp: field_map_emails,field2,field3 ..'),
    ];
    $form['ses_webform_handler_exlude'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Exclude Webform Handlers'),
      '#default_value' => $config->get('ses_webform_handler_exlude'),
      '#description' => $this->t('Exclude webform handler, exp: handler1,handler2....'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('ses_email_bounce.settings')
      ->set('ses_api_status', $form_state->getValue('ses_api_status'))
      ->set('header_content_type', $form_state->getValue('header_content_type'))
      ->set('header_x_amz_sns_topic_arn', $form_state->getValue('header_x_amz_sns_topic_arn'))
      ->set('base_domain', $form_state->getValue('base_domain'))
      ->set('ses_subscribtion_log_relative_path', $form_state->getValue('ses_subscribtion_log_relative_path'))
      ->set('ses_notification_log_relative_path', $form_state->getValue('ses_notification_log_relative_path'))
      ->set('ses_node_map', $form_state->getValue('ses_node_map'))
      ->set('ses_webform_handler_exlude', $form_state->getValue('ses_webform_handler_exlude'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
